/***************************************************************************
 *
 *  $Id: kzenmediafiles.cpp,v 1.12 2005/06/01 08:04:20 muszilla Exp $
 *
 *  Copyright (C) 2005 by Andreas Mussgiller
 *  muszilla@users.sourceforge.net
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the
 *  Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 ***************************************************************************/

#include <iostream>

#include <qdir.h>
#include <qregexp.h>
#include <qfileinfo.h>

#include <kmimetype.h>
#include <klocale.h>
#include <kfilemetainfo.h>
#include <kdebug.h>

#include <fileref.h>
#include <tag.h>

#include "kzenconfig.h"
#include "kzenmediafiles.h"
#include "kzentrack.h"

bool KZenMediaFiles::isMediaFile(const KURL & url)
{
  return isMP3(url);
}

bool KZenMediaFiles::isPlaylistFile(const KURL & url)
{
  KMimeType::Ptr result = KMimeType::findByURL(url, 0);
  return (result->name() == "audio/mpegurl" ||
	  result->name() == "audio/x-mpegurl");
}

bool KZenMediaFiles::isMP3(const KURL & url)
{
  KMimeType::Ptr result = KMimeType::findByURL(url, 0);
  return result->name() == "audio/x-mp3";
}

bool KZenMediaFiles::isWMA(const KURL & url)
{
  KMimeType::Ptr result = KMimeType::findByURL(url, 0);
  return result->name() == "audio/x-ms-wma";
}

bool KZenMediaFiles::isWAV(const KURL & url)
{
  KMimeType::Ptr result = KMimeType::findByURL(url, 0);
  return result->name() == "audio/x-wav";
}

bool KZenMediaFiles::isDirectory(const KURL & url)
{
  QFileInfo info(url.path());
  if (info.exists() && info.isDir()) return true;
  return false;
}

KZenTrack * KZenMediaFiles::createTrackMP3(const KURL & url)
{
  TagLib::FileRef f(url.path());
  if(f.isNull() || !f.tag()) return 0;

  KZenTrack * newTrack = new KZenTrack();
  newTrack->setCodec(KZenTrack::CodecMP3);

  QFile file(url.path());
  newTrack->setFileSize(file.size());

  TagLib::Tag * tag = f.tag();
  QString temp;

  temp = TStringToQString(tag->artist());
  if (temp.length()>0) {
    newTrack->setArtist(temp);
  } else {
    newTrack->setArtist(i18n("Unknown Artist"));
  }
    
  temp = TStringToQString(tag->album());
  if (temp.length()>0) {
    newTrack->setAlbum(temp);
  } else {
    newTrack->setAlbum(i18n("Unknown Album"));
  }
  
  temp = TStringToQString(tag->title());
  if (temp.length()>0) {
    newTrack->setTitle(temp);
  } else {
    newTrack->setTitle("");
  }
    
  temp = TStringToQString(tag->genre());
  if (temp.length()>0) {
    newTrack->setGenre(temp);
  } else {
    newTrack->setGenre("Other");
  }
  
  newTrack->setTrackNumber(tag->track());
  newTrack->setYear(tag->year());
  
  TagLib::AudioProperties * properties = f.audioProperties();
  
  newTrack->setLength(properties->length());
  
  QString filename = url.path();
  filename = filename.right(filename.length()-filename.findRev(QDir::separator(),-1)-1);
  newTrack->setFileName(filename);
  
  if (newTrack->getTitle().length()==0) {
    while ((filename.right(4)).lower()==".mp3")
      filename = filename.left(filename.length()-4);
    
    newTrack->setTitle(filename);
  }
  
  return newTrack;
}

bool KZenMediaFiles::checkLibraryPath(const KURL & url)
{
  QString path = url.path();
  QStringList list = QStringList::split("/",path);
  
  QDir dir;
  path = "";
  for (QStringList::Iterator it=list.begin();it!=list.end();++it) {
    path += "/";
    path += *it;
    
    QFileInfo info(path);
    if (!info.exists()) {
      if (!dir.mkdir(path)) {
	return false;
      }
    }
    
    dir.cd(path);
  }
  
  return true;
}

KURL KZenMediaFiles::saveFilename(KZenTrack * track, const KURL & surl, bool & valid)
{
  KURL url = surl;
  
  QString tracknumber;
  tracknumber.sprintf("%02d",track->getTrackNumber());

  QString artist = track->getArtist();
  artist.replace("/","_");
  QString album = track->getAlbum();
  album.replace("/","_");
  QString title = track->getTitle();
  title.replace("/","_");

  QString path = KZenConfig::get()->savePath();
  path.replace("%artist",artist,false);
  path.replace("%album",album,false);
  path.replace("%title",title,false);
  path.replace("%track",tracknumber,false);
  path.replace(" ","_");
  path.replace("'","_");
  path.replace("__","_");
  url.addPath(path.lower());

  if (!checkLibraryPath(url)) {
    valid = false;
    return url;
  }
  
  QFileInfo info; 
  info.setFile(url.path());
  if (!info.isWritable()) {
    valid = false;
    return url;
  }

  QString codecname = track->getCodecName();
  codecname = codecname.lower();
  QString name = KZenConfig::get()->saveName();
  name.replace("%artist",artist,false);
  name.replace("%album",album,false);
  name.replace("%title",title,false);
  name.replace("%track",tracknumber,false);
  name.replace("/","_");
  name.replace(" ","_");
  name.replace("'","_");
  name.replace("__","_");
  url.addPath(name.lower());
  
  int count = 0;
  name.sprintf("%s.%s",url.path().latin1(),codecname.latin1());
  while (QFile::exists(name)) {
    count++;
    name.sprintf("%s_%d.%s",url.path().latin1(),count,codecname.latin1());
  }
  
  url.setPath(name);
  
  valid = true;
  
  return url;
}
